# =============================================================================
# Copyright (c) 2018-2019 gardner ag. All Rights Reserved.
#
# This software is the confidential and proprietary information of gardner ag
# ("Confidential Information"). You shall not disclose such 
# Confidential Information and shall use it only in
# accordance with the terms of the license agreement you entered into
# with gardner ag.
#
# GARDNER AG MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
# SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT. GARDNER AG SHALL NOT BE LIABLE FOR ANY DAMAGES
# SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
# THIS SOFTWARE OR ITS DERIVATIVES.
# =============================================================================
import cherrypy
from cherrypy.process import wspbus, plugins

class Plugin(plugins.SimplePlugin):
    def __init__(self, bus, klass):
        plugins.SimplePlugin.__init__(self, bus)
        self.p = klass()

    def start(self):
        self.bus.log('Starting up ' + self.p.logname)
        self.bus.subscribe(self.p.name + "/start", self.startProjectCode)

    def stop(self):
        self.bus.log('Stopping ' +  self.p.logname)
        self.bus.unsubscribe(self.p.name + "/start", self.startProjectCode)
        self.stopProjectCode()

    def startProjectCode(self, options):
        self.p.start(options)

    def stopProjectCode(self):
      invert_op = getattr(self.p, "stop", None)
      if callable(invert_op):
          self.p.stop()




# =============================================================================
# Copyright (c) 2018-2019 gardner ag. All Rights Reserved.
#
# This software is the confidential and proprietary information of gardner ag
# ("Confidential Information"). You shall not disclose such 
# Confidential Information and shall use it only in
# accordance with the terms of the license agreement you entered into
# with gardner ag.
#
# GARDNER AG MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
# SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT. GARDNER AG SHALL NOT BE LIABLE FOR ANY DAMAGES
# SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
# THIS SOFTWARE OR ITS DERIVATIVES.
# =============================================================================
import cherrypy as applyai
from cherrypy.process.plugins import SimplePlugin
import os
import datetime
import re
import paho.mqtt.client as mqtt
import logging
import logging.config

applyai._cplogging.LogManager.time = lambda self : '[' + datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S,%f")[:23] + ']'
applyai._cplogging.LogManager.access_log_format = ( '{t} WEBGUI {h} "{r}" {s} {b}' ) #"{f}" "{a}"'  )
#applyai._cplogging.LogManager(logger_root = "applyai")

class Logs(object):
  def __init__(self):
    self.perPage = 20

  exposed = True
  def GET(self, page=1):
    page = int(page)
    if page < 1:
      page = 1
    start  = (page-1) * self.perPage
    finish = start + self.perPage

    filename  = '../common/logs/error.log'
    # html = '<table id="logtable" class="table">'
    html = '<tr><th>timestamp</th><th>source</th><th>message</th></tr>\n'
    with open(filename) as fp:
      idx = 0
      for line in self.reversed_fp_iter(fp, 256):
        if idx >= start:
          line = line.replace('\n','')
          s = line.split(' ')
          if len(s) > 2:
            html += '<tr>'
            html += '<td style="padding: 0px;white-space: nowrap;">' + s[0].replace('[','') + ' ' + s[1].replace(']','') + '</td>' 
            html += '<td style="padding: 0px">' + s[2] + '</td>'
            tmp = ''
            for i in range(3,len(s)):
              tmp += s[i] + ' '
            html += '<td style="padding: 0px">' + tmp + '</td>' 
            html += '</tr>'

        idx += 1
        if idx > finish:
          break
    #html += '</table>'
    applyai.response.headers['Content-Type'] = "text/html"
    return html

  def reversed_fp_iter(self, fp, buf_size=1024):
      """a generator that returns the lines of a file in reverse order
      ref: https://stackoverflow.com/a/23646049/8776239
      """
      segment = None  # holds possible incomplete segment at the beginning of the buffer
      offset = 0
      fp.seek(0, os.SEEK_END)
      file_size = remaining_size = fp.tell()
      while remaining_size > 0:
          offset = min(file_size, offset + buf_size)
          fp.seek(file_size - offset)
          buffer = fp.read(min(remaining_size, buf_size))
          remaining_size -= buf_size
          lines = buffer.splitlines(True)
          # the first line of the buffer is probably not a complete line so
          # we'll save it and append it to the last line of the next buffer
          # we read
          if segment is not None:
              # if the previous chunk starts right from the beginning of line
              # do not concat the segment to the last line of new chunk
              # instead, yield the segment first
              if buffer[-1] == '\n':
                  #print 'buffer ends with newline'
                  yield segment
              else:
                  lines[-1] += segment
                  #print 'enlarged last line to >{}<, len {}'.format(lines[-1], len(lines))
          segment = lines[0]
          for index in range(len(lines) - 1, 0, -1):
              if len(lines[index]):
                  yield lines[index]
      # Don't yield None if the file was empty  hhh
      if segment is not None:
          yield segment

class LogfilePlugin(SimplePlugin):
  
  def __init__(self, bus):
    SimplePlugin.__init__(self, bus)
    self.name = "logs"
    self.logname = self.name.upper().ljust(6)
    self.client = None
    applyai.log('in init', self.logname)

    # grab the current LogRecordFactory
    self.old_factory = logging.getLogRecordFactory()

  def start(self):
    '''Called when the engine starts'''
    # create a mqtt client
    self.client = mqtt.Client()
    self.client.on_connect = self.on_connect
    self.client.connect("localhost", 1883, 60)
    logging.setLogRecordFactory(self.record_factory)
    applyai.log('Starting up MQTT client' + self.logname, self.logname)
    #self.client.loop_forever(timeout=1.0, max_packets=1, retry_first_connection=False)
    self.client.loop_start()

  def stop(self):
    '''Called when the engine stops'''
    if self.client != None:
      self.client.loop_stop(force=False)
      self.client.disconnect()
    applyai.log('Stopping ' + self.logname, self.logname)

  def on_connect(self, client, userdata, flags, rc):
    applyai.log("Connected with result code "+str(rc), self.logname)

    # Subscribing in on_connect() means that if we lose the connection and
    # reconnect then subscriptions will be renewed.
    client.subscribe("$SYS/#")

  # add mqtt publish to the current LogRecordFactory
  # this will catch all logs and send them on to the mqtt broker
  def record_factory(self, *args, **kwargs):
    record = self.old_factory(*args, **kwargs)
    logstr = self.replacenth(record.getMessage(), ' ', '@', 2)
    logstr = self.replacenth(logstr, ' ', '@', 2).replace('@  ','@').replace('@ ','@')
    self.client.publish('logger/message', logstr)
    return record

  def replacenth(self, string, sub, wanted, n):
    where = [m.start() for m in re.finditer(sub, string)][n-1]
    before = string[:where]
    after = string[where:]
    after = after.replace(sub, wanted, 1)
    newString = before + after
    return newString



# ----------------------------------------------------------------------------------------------------
# --- Python_3 ---                                                             Stand: 21.03.2019 / LG
# ----------------------------------------------------------------------------------------------------
#
# ----------------------------------------------------------------------------------------------------
import numpy as np
import json
import pandas as pd
import math
import time
import os

#print(pd.__name__)
#print(np.__name__)

class stats:
    def __init__(self):
        self.timespath = '../common/stats/times.txt'
        self.times = pd.DataFrame(columns=['starttime','endtime','cycletime','flag'])
        self.maxCycles = 10
        self.jsonpath = '../common/stats/stats.json'
        self.json_data = {}
        try: 
            self.times = pd.read_csv('../common/stats/times.txt')
        except:
            print('no times.txt file')

    def insertCycletime(self, flag):
        zeit = pd.Timestamp.now()
        if flag == 1: #insert starttime
            self.times = self.times.append({'starttime': zeit, 'flag': flag}, ignore_index=True)
        if flag == 2: #insert endtime
            self.times.at[self.times.index[-1], 'endtime']   = zeit
            self.times.at[self.times.index[-1], 'cycletime'] = (self.times.at[self.times.index[-1], 'endtime'] - self.times.at[self.times.index[-1], 'starttime']).total_seconds()
            self.times.at[self.times.index[-1], 'flag']      = flag

        #print(self.times)

    def updateCycletime(self):
        #print(self.times["cycletime"])
        cycleTime = self.times["cycletime"].tail(10).mean()
        data = self.json_data['Stats']
        for i in range(len(data)):
            #print(data[i]['name'])
            if data[i]['name'] == "cycleTime":
                data[i]['value'] = cycleTime
        self.times.to_csv('../common/stats/times.txt', index=False, float_format='%.3f')

    def cycleTime(self):
        #df = pd.read_csv(self.times)
        #print(df["time"].mean())
        df_ = pd.DataFrame(columns=['time'])
        df_.append({'time': 2}, ignore_index=True)
        #print(df_)

    def incrementDetection(self, n):
        #print(json_data['Stats'])
        data = self.json_data['Stats']
        for i in range(len(data)):
            #print(data[i]['name'])
            if data[i]['name'] == "nDetects":
                data[i]['value'] = str(int(data[i]['value']) + n)
                #print(data[i]['name'])
                #print(data[i]['value'])
        #print(self.json_data)

    def read(self):
        with open(self.jsonpath, 'r') as file:
            self.json_data = json.load(file)

    def write(self):
        data = json.dumps(self.json_data, indent=4, sort_keys=True)
        with open(self.jsonpath, 'w') as f:
            f.write(data)

if __name__ == "__main__":
    stats = stats()
    stats.read()
    incr = stats.incrementDetection(1)
    stats.cycleTime()
    stats.insertCycletime(1)
    time.sleep(2)
    stats.insertCycletime(2)
    stats.updateCycletime()
    stats.write()
    print("stats.py | end")



# =============================================================================
# Copyright (c) 2018-2019 gardner ag. All Rights Reserved.
#
# This software is the confidential and proprietary information of gardner ag
# ("Confidential Information"). You shall not disclose such 
# Confidential Information and shall use it only in
# accordance with the terms of the license agreement you entered into
# with gardner ag.
#
# GARDNER AG MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
# SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT. GARDNER AG SHALL NOT BE LIABLE FOR ANY DAMAGES
# SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
# THIS SOFTWARE OR ITS DERIVATIVES.
# =============================================================================
import cherrypy as applyai
import json

class Config(object):
  def __init__(self):
    self.cfg = {}

  @applyai.expose
  def getSettingsForm(self):
    tab = "<table>"
    for c in self.cfg["settings"]:
      for v in self.cfg["settings"][c]["list"]:
        tab += "<tr>"
        tab += "<td>" + c
        tab += "</td>"
        tab += "<td>" + v
        tab += "</td>"
        if "text" in self.cfg["settings"][c][v]:
          tab += "<td>" + str(self.cfg["settings"][c][v]["text"])
          tab += "</td>"
          tab += "<td><input size='4' value='" + str(self.cfg["settings"][c][v]["value"]) + "'/>"
          tab += "</td>"
        else: # must be an array
          for a in self.cfg["settings"][c][v]:
            tab += "<tr><td></td><td></td><td></td>"
            for attr, value in a.items():
              tab += "<td>" + str(attr)
              tab += "</td>"
              tab += "<td><input size='4' value='" + str(value) + "'/>"
              tab += "</td>"
            tab += "</tr>"
        tab += "</tr>"
    tab += "</table>"
    
    applyai.response.headers['Content-Type'] = "text/html"
    return tab

  @applyai.expose
  def updateSettingVar(self, plugin, var, value):
    self.cfg["settings"][plugin][var]["value"] = value
    return self.cfg["settings"][plugin]

  @applyai.expose
  @applyai.tools.json_out()
  def getConfig(self, page=""):
    if page == "":
      return self.cfg["settings"]
    else:
      try:
        return self.cfg["settings"][page]
      except:
        return "that did not work ['settings']['" + page + "']"
    return self.cfg



# =============================================================================
# Copyright (c) 2018-2019 gardner ag. All Rights Reserved.
#
# This software is the confidential and proprietary information of gardner ag
# ("Confidential Information"). You shall not disclose such 
# Confidential Information and shall use it only in
# accordance with the terms of the license agreement you entered into
# with gardner ag.
#
# GARDNER AG MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
# SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT. GARDNER AG SHALL NOT BE LIABLE FOR ANY DAMAGES
# SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
# THIS SOFTWARE OR ITS DERIVATIVES.
# =============================================================================
import cherrypy as applyai
import requests
import json
import os
import cv2
import time
from datetime import timedelta
import applyai_vision.job as job
import applyai_vision.memStore as ms
import pandas as pd
from pandas.io.json import json_normalize

class stdAPI:

  def __init__(self, name):
    self.logname = name[:6].upper().ljust(6)
    self.name = name
    self.store = ms.memStore(name)
    self.project = os.environ['PROJECT']
    self.cfgFilename = './projects/' + self.project + '/config/' + self.name + '.conf'
    self.cfg = {}
    applyai.log('init API',self.logname)

  #Converts the input JSON to a DataFrame
  def convertToDF(self,dfJSON):
      return(json_normalize(dfJSON))

  #Converts the input DataFrame to JSON 
  def convertToJSON(self, df):
      resultJSON = df.to_json(orient='records')
      return(resultJSON)

  @applyai.expose
  @applyai.tools.json_in()
  @applyai.tools.json_out()
  def start(self): #, sequence='detect', frameIn='', channel=0, targets="[]"):

    applyai.log('$Start', self.name)

    params = applyai.request.json
    params['name'] = self.name             # always overwrite this

    if 'sequence' not in params:          # default sequence detect
      params['sequence'] = 'detect'

    if 'frameIn' not in params:
      params['frameIn'] = self.name

    if 'frameOut' not in params:
      params['frameOut'] = self.name

    if 'channel' not in params:
      params['channel'] = 0

    # in image of this plugin = out image of that plugin
    self.store.updateFrameIn(self.name, 0, self.store.fetchFrameOut(params['frameIn'],0))

    targets = pd.DataFrame(columns=['plugin'])
    if 'targets' in params:
      targets = self.convertToDF(params['targets'])

    params['targets'] = targets
    #applyai.log(str(params), self.logname)

    targets = self.main(params)

    params['targets'] = targets.to_dict('records')
    params['frameIn'] = self.name       # use this to save the last plugin in frameIn for the nect plugin
    #applyai.log(str(params), self.logname)
    applyai.log('$Finished', self.name)
    return params

  @applyai.expose
  def getFrame(self, frame, camera=0, rand=0):
    # applyai.log('getFrame ' + str(frame) + '|' + str(camera), self.logname)
    camera = int(camera)
    if frame == 'in':
      return(self.getFrameIn(camera, rand))
    else:
      return(self.getFrameOut(camera, rand))

  @applyai.expose
  def getFrameIn(self, camera=0, rand=0):
    applyai.response.headers['Content-Type'] = "image/jpg"
    return cv2.imencode('.jpg', self.store.fetchFrameIn(self.name, camera))[1].tobytes()

  @applyai.expose
  def getFrameOut(self, camera=0, rand=0):
    applyai.response.headers['Content-Type'] = "image/jpg"
    return cv2.imencode('.jpg', self.store.fetchFrameOut(self.name, camera))[1].tobytes()

  @applyai.expose
  @applyai.tools.json_out()
  def getTargets(self):
    applyai.response.headers['Content-Type'] = "application/json"
    #jstr = '{"targets":' + self.store.fetchTargets(self.name).to_json(orient='records') + '}'
    return self.store.fetchTargets(self.name).to_json(orient='records')

  @applyai.expose
  @applyai.tools.json_out()
  def getConfig(self):
    with open(self.cfgFilename) as json_file:
      self.cfg = json.load(json_file)

    return(self.cfg)

  def exportConfigVariable(self, typ, valueType, var, defaultValue, units, guiText, guiLongText, select=""):
    self.cfg[var] = {}
    self.cfg[var]['type'] = typ
    self.cfg[var]['valueType'] = valueType
    self.cfg[var]['value'] = defaultValue
    self.cfg[var]['units'] = units
    self.cfg[var]['guiText'] = guiText
    self.cfg[var]['guiLongText'] = guiLongText
    self.cfg[var]['select'] = select

  def updateConfig(self):
    # load old config from file in stored
    stored = self.loadConfig()
    for c in self.cfg:
      if not c in stored:
        stored[c] = self.cfg[c]
      for a in self.cfg[c]:
        if self.cfg[c][a] != stored[c][a]:
          self.cfg[c][a] = stored[c][a]

    return self.cfg

  def monitorConfig(self):
    stamp = os.stat(self.cfgFilename).st_mtime
    if stamp != self.cached_stamp:
        self.cached_stamp = stamp
        applyai.log('cfg file modified', self.logname)
        self.cfg = self.updateConfig()
        applyai.engine.publish(self.name + '/monitorConfig', self.cfg)

  def loadConfig(self):
    if os.path.isfile(self.cfgFilename):
      with open(self.cfgFilename) as json_file:
        cfg = json.load(json_file)
      return(cfg)
    return({})

  def saveConfigToFile(self):
    with open(self.cfgFilename, 'w') as outfile:
      outfile.write(json.dumps(self.cfg, indent=2))

  @applyai.expose
  @applyai.tools.json_in()
  @applyai.tools.json_out()
  def saveConfig(self):
    with open(self.cfgFilename, 'w') as outfile:
      outfile.write(json.dumps(applyai.request.json, indent=2))
      return({'info':'updated cfg ' + self.cfgFilename})
    return({'info':'Project file cannot be updated'})
    
  #@applyai.expose
  #def getFrame(self, frame='in', rand=''):

  #  application = applyai.tree.apps['/Project']
  #  plugins = application.config['config']['plugins']

  #  for p in plugins:
  #    if p['name'] == self.name:
  #      path = str('../common/images/frame_%03d_%02d.jpg' % (p['frameIn'], p['camera']))
  #      if frame == 'out':
  #        path = str('../common/images/frame_%03d_%02d.jpg' % (p['frameOut'], p['camera']))
  #      
  #      with open(path, 'rb') as f:
  #        img = f.read()
  #
  #      applyai.response.headers['Content-Type'] = "image/jpg"
  #      return img
  #  
  #  return("no image")
  @applyai.expose
  def index(self):
    html = self.html_header()
    html += '<body onload="init()">'
    html += '<section class="section">'
    html += '<div class="container">'
    html += self.html_name()
    html += '<hr>'
    html += self.html_config()
    html += '<hr>'
    html += self.html_frames()
    html += '<hr>'
    html += self.html_buttons()
    html += '</section></div></body></html>'
    return(html)

  def html_header(self):
    html = '<!DOCTYPE html><html>\n'
    html += '<head>\n'
    html += '<meta charset="utf-8"><meta name="viewport" content="width=device-width, initial-scale=1">\n'
    html += '<title>applyai Plugin Tester</title>\n'
    html += '<style>body {font-family:arial;}</style>\n'
    html += '<link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bulma@0.8.0/css/bulma.min.css"/>\n'
    html += self.getJS()
    html += '</head>\n'
    return(html)

  def getJS(self):
    js = '<script type="text/javascript">\n'
    js += 'function init() {\n'
    js += '}\n'
    js += 'function startTest() {\n'
    js += ' get("/' + self.name + '/start", function() {\n'
    js += '   document.getElementById("frameOut").src = "/' + self.name + '/getFrameOut?camera=0&rand=" + Math.random()\n'
    js += ' })\n'
    js += '}\n'
    js += 'function get(what, next) {\n'
    js += 'xhr = new XMLHttpRequest()\n'
    js += 'xhr.open("GET", what)\n'
    js += 'xhr.onload = function() {\n'
    js += ' if (xhr.status === 200) {\n'
    js += '   console.log(xhr.responseText)\n'
    js += '   next()\n'
    js += ' } else {\n'
    js += '   alert("Request failed.  Returned status of " + xhr.status)\n'
    js += ' }\n'
    js += '}\n'
    js += 'xhr.send()\n'
    js += '}\n'
    js += '</script>\n'
    return(js)

  def html_name(self):
    html =  '<div><h2 class="title">applyai Plugin Tester</h2></div>\n'
    html += '<div><h2 class="title">Plugin: ' + self.name + '</h2></div>\n'
    return(html)

  def html_config(self):
    cfg = self.getConfig()
    html = '<div>\n'
    html += 'Description of Plugin: ' + cfg['description']['guiLongText']
    html += '</div>\n'
    return(html)

  def html_frames(self):
    html = '<div><figure class="image is-4by3">\n'
    html += '<img id="frameIn" src="/' + self.name + '/getFrameIn">\n'
    html += '</figure></div>\n'
    html += '<hr>\n'
    html += '<div><figure class="image is-4by3">\n'
    html += '<img id="frameOut" src="/' + self.name + '/getFrameOut">\n'
    html += '</figure></div>\n'
    return(html)

  def html_buttons(self):
    html = '<div>\n'
    html += '<button class="button is-primary" onclick="startTest()">Test</button>\n'
    html += '</div>\n'
    return(html)


class stdPCode:
  
  def __init__(self, name):

    self.logname = name[:6].upper().ljust(6)
    self.name = name
    self.project = os.environ['PROJECT']
    self.cfgFilename = './projects/' + self.project + '/config/' + self.name + '.conf'
    self.cfg = {}
    self.store = ms.memStore(name)

    if not os.path.isfile(self.cfgFilename):
      with open(self.cfgFilename, 'w') as outfile:
        outfile.write('{}')

    self.cached_stamp = os.stat(self.cfgFilename).st_mtime
    self.monitor = job.Job(interval=timedelta(seconds=1), execute=self.monitorConfig)
    self.monitor.start()
    
    applyai.log('init PCode',self.logname)

  def exportConfigVariable(self, typ, valueType, var, defaultValue, units, guiText, guiLongText, select=""):
    self.cfg[var] = {}
    self.cfg[var]['type'] = typ
    self.cfg[var]['valueType'] = valueType
    self.cfg[var]['value'] = defaultValue
    self.cfg[var]['units'] = units
    self.cfg[var]['guiText'] = guiText
    self.cfg[var]['guiLongText'] = guiLongText
    self.cfg[var]['select'] = select

  def updateConfig(self):
    # load old config from file in stored
    stored = self.loadConfig()
    for c in self.cfg:
      if not c in stored:
        stored[c] = self.cfg[c]
      for a in self.cfg[c]:
        if self.cfg[c][a] != stored[c][a]:
          self.cfg[c][a] = stored[c][a]
    return self.cfg

  def monitorConfig(self):
    stamp = os.stat(self.cfgFilename).st_mtime
    if stamp != self.cached_stamp:
        self.cached_stamp = stamp
        applyai.log('cfg file modified', self.logname)
        self.cfg = self.updateConfig()
        applyai.engine.publish(self.name + '/monitorConfig', self.cfg)

  def loadConfig(self):
    if self.name == 'Project':
      cfg = {}
      cfg['list'] = self.application.config['config']['list']
      for c in cfg['list']:
        cfg[c] = self.application.config['config'][c]
    else:
      with open(self.cfgFilename) as json_file:
        cfg = json.load(json_file)
    return(cfg)

  def saveConfig(self):
    with open(self.cfgFilename, 'w') as outfile:
      outfile.write(json.dumps(self.cfg, indent=2))

  def stop(self):
    self.monitor.stop()
    applyai.log('Stopping monitor', self.logname)

  def loadCfgList(self):
    if hasattr(self, 'application'):
      return(self.application.config['config']['list'])
    else:
      application = applyai.tree.apps['/' + self.name]
      return(application.config['config']['list'])
    return([])

  def loadValue(self, valueStr):
    if hasattr(self.cfg, 'application'):
      for value in self.application.config['config']['list']:
        if value == valueStr:
          return self.application.config['config'][value]['value']
      return self.application.config['config']['list']
    else:
      application = applyai.tree.apps['/' + self.name]
      for value in application.config['config']['list']:
        if value == valueStr:
          return application.config['config'][value]['value']
      return application.config['config']['list']
    return('')




# =============================================================================
# Copyright (c) 2018-2019 gardner ag. All Rights Reserved.
#
# This software is the confidential and proprietary information of gardner ag
# ("Confidential Information"). You shall not disclose such 
# Confidential Information and shall use it only in
# accordance with the terms of the license agreement you entered into
# with gardner ag.
#
# GARDNER AG MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
# SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT. GARDNER AG SHALL NOT BE LIABLE FOR ANY DAMAGES
# SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
# THIS SOFTWARE OR ITS DERIVATIVES.
# =============================================================================
import cherrypy as applyai
import os

def getLogname():
  name = os.path.basename(__file__).replace('pluginProject','').replace('.py','')
  return name

if __name__ == "__main__":
  print(getLogname())


# =============================================================================
# Copyright (c) 2018-2019 gardner ag. All Rights Reserved.
#
# This software is the confidential and proprietary information of gardner ag
# ("Confidential Information"). You shall not disclose such 
# Confidential Information and shall use it only in
# accordance with the terms of the license agreement you entered into
# with gardner ag.
#
# GARDNER AG MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
# SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT. GARDNER AG SHALL NOT BE LIABLE FOR ANY DAMAGES
# SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
# THIS SOFTWARE OR ITS DERIVATIVES.
# =============================================================================
import threading, time, signal
from datetime import timedelta

class Job(threading.Thread):
  def __init__(self, interval, execute, *args, **kwargs):
    threading.Thread.__init__(self)
    self.daemon = False
    self.stopped = threading.Event()
    self.interval = interval
    self.execute = execute
    self.args = args
    self.kwargs = kwargs
      
  def stop(self):
    self.stopped.set()
    self.join()

  def run(self):
    while not self.stopped.wait(self.interval.total_seconds()):
      self.execute(*self.args, **self.kwargs)

class ProgramKilled(Exception):
    pass

def signal_handler(signum, frame):
    raise ProgramKilled

def foo():
    print(time.ctime())

WAIT_TIME_SECONDS = 1

if __name__ == "__main__":
    signal.signal(signal.SIGTERM, signal_handler)
    signal.signal(signal.SIGINT, signal_handler)
    job = Job(interval=timedelta(seconds=WAIT_TIME_SECONDS), execute=foo)
    job.start()
    
    while True:
          try:
              time.sleep(1)
          except ProgramKilled:
              print("Program killed: running cleanup code")
              job.stop()
              break



import pandas as pd
import numpy as np
import cv2

class memStore:

  store = {}
  maxCam = 4

  def __init__(self, name):
    self.add(name)

  def add(self, name):
    if name not in memStore.store:
      memStore.store[name] = {}
      memStore.store[name]['name'] = name
      memStore.store[name]['targets']  = pd.DataFrame(columns=['plugin'])
      memStore.store[name]['frameIn']  = []
      memStore.store[name]['frameOut'] = []
      for i in range(memStore.maxCam):
        img = self.create_blank(600,100)
        memStore.store[name]['frameIn'].append(img)
        memStore.store[name]['frameOut'].append(img.copy())
  
  def updateFrameIn(self, name, cam, frameIn):
    if name in memStore.store:
      if cam < memStore.maxCam:
        memStore.store[name]['frameIn'][cam] = frameIn
    else:
      self.add(name)
      if cam < memStore.maxCam:
        memStore.store[name]['frameIn'][cam] = frameIn

  def updateFrameOut(self, name, cam, frameOut):
    if name in memStore.store:
      if cam < memStore.maxCam:
        memStore.store[name]['frameOut'][cam] = frameOut
    else:
      self.add(name)
      if cam < memStore.maxCam:
        memStore.store[name]['frameOut'][cam] = frameIn

  def updateTargets(self, name, targets):
    if name in memStore.store:
      memStore.store[name]['targets']  = targets

  def fetch(self, name):
    if name in memStore.store:
      return memStore.store[name]
    return {}

  def fetchTargets(self, name):
    if name in memStore.store:
      return memStore.store[name]['targets']
    return {}

  def fetchFrameIn(self, name, camera=0):
    if name in memStore.store:
      camera = int(camera)
      if camera < len(memStore.store[name]['frameIn']):
        return memStore.store[name]['frameIn'][camera]
    return self.create_blank(600,100)

  def fetchFrameOut(self, name, camera=0):
    if name in memStore.store:
      camera = int(camera)
      if camera < len(memStore.store[name]['frameOut']):
        return memStore.store[name]['frameOut'][camera]
    return self.create_blank(600,100)

  def print(self):
    for s in memStore.store:
      print(memStore.store[s]['name'])

  def create_blank(self, width, height, rgb_color=(0, 0, 0)):
    """Create new image(numpy array) filled with certain color in RGB"""
    # Create black blank image
    image = np.zeros((height, width, 3), np.uint8)

    # Since OpenCV uses BGR, convert the color first
    color = tuple(reversed(rgb_color))
    # Fill image with color
    image[:] = color

    cv2.putText(image, "ERROR check image source!", (20, 50), cv2.FONT_HERSHEY_SIMPLEX, 1.1, (255,255,255), 3)
    return image

if __name__ == "__main__":
  
  targets  = pd.DataFrame(columns=['plugin'])
  frameIn  = []
  frameOut = []
  frameIn.append(255 * np.ones(shape=[512, 512, 3], dtype=np.uint8))
  frameOut.append(255 * np.ones(shape=[512, 512, 3], dtype=np.uint8))

  ms = memStore('Camera')
  ms.add('Mask')
  ms.add('Model')

  o = ms.fetch('Calc')

  ms.print()
